import shutil
from typing import Any, Optional, List

from pydantic import BaseModel, Field

from wexample_prompt.const.colors import COLOR_RESET
from wexample_prompt.mixins.with_indent import WithIndent
from wexample_prompt.themes.default.abstract_prompt_theme import AbstractPromptTheme
from wexample_prompt.themes.default.default_prompt_theme import DefaultPromptTheme
from wexample_prompt.utils.prompt_response import PromptResponse
from wexample_prompt.utils.prompt_response_line import PromptResponseLine


class IOManager(BaseModel, WithIndent):
    theme: AbstractPromptTheme = Field(
        default_factory=DefaultPromptTheme,
        description="Allow to customize colors")
    _tty_width: int = shutil.get_terminal_size().columns

    def print_separator(self, message: Optional[str] = None, char: str = "_"):
        self.print(message.ljust(self._tty_width, char))

    def print_responses(self, responses: List[PromptResponse]) -> None:
        for response in responses:
            self.print_response(response)

    def print_response(self, response: PromptResponse) -> None:
        for line in response.lines:
            self.print_response_line(line, response)

    def print_response_line(self, line: PromptResponseLine, response: PromptResponse) -> None:
        if line.line_type:
            color_type = line.line_type
        else:
            color_type = response.message_type

        self.print(f'{self.build_indent()}{self.theme.get_color(color_type)}{line.message}{COLOR_RESET}')

    @staticmethod
    def print(message: Any, **kwargs: Any) -> None:
        print(message, **kwargs)



from abc import ABC, abstractmethod

from pydantic import BaseModel

from wexample_prompt.enums.message_type import MessageType


class AbstractPromptTheme(BaseModel, ABC):
    @abstractmethod
    def get_color(self, message_type: MessageType) -> str:
        pass



from typing import Optional

from pydantic import BaseModel

from wexample_prompt.const.types import LineDict
from wexample_prompt.enums.message_type import MessageType


class PromptResponseLine(BaseModel):
    message: str
    line_type: Optional[MessageType] = None

    @staticmethod
    def from_dict(line_dict: LineDict) -> 'PromptResponseLine':
        return PromptResponseLine(
            message=line_dict.get('message'),
            line_type=line_dict.get('type'),
        )

    @staticmethod
    def from_message(
        message: str,
        line_type: Optional[str] = None
    ) -> 'PromptResponseLine':
        return PromptResponseLine(
            message=message,
            line_type=line_type,
        )



import os
from typing import Optional, List, Dict

from pydantic import BaseModel

from wexample_prompt.const.types import LineDict
from wexample_prompt.enums.message_type import MessageType
from wexample_prompt.utils.prompt_response_line import PromptResponseLine


class PromptResponse(BaseModel):
    lines: List[PromptResponseLine] = []
    message_type: MessageType

    def __init__(self, message: Optional[str] = None, message_type: MessageType = MessageType.LOG) -> None:
        super().__init__(message=message, message_type=message_type)

        if message:
            self.lines = self.split_lines(message)

    @staticmethod
    def split_lines(message: str) -> List[PromptResponseLine]:
        lines = message.split(os.linesep)
        return [PromptResponseLine(message=message) for message in lines]

    @staticmethod
    def from_lines(messages: List[str]) -> 'PromptResponse':
        response = PromptResponse()

        for message in messages:
            response.lines.append(PromptResponseLine.from_message(message=message))

        return response

    @staticmethod
    def from_multiline_message(message: str) -> 'PromptResponse':
        response = PromptResponse()
        response.lines = response.split_lines(message)

        return response

    @staticmethod
    def from_dicts(lines: List[LineDict]) -> 'PromptResponse':
        response = PromptResponse()

        for line_dict in lines:
            response.lines.append(PromptResponseLine.from_dict(line_dict))

        return response



from setuptools import setup, find_packages

setup(
    name='wexample-prompt',
    version=open('version.txt').read(),
    author='weeger',
    author_email='contact@wexample.com',
    description='Helper for your tty interactions',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/wexample/python-prompt',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        'pydantic',
        'rich',
    ],
    python_requires='>=3.6',
)



from wexample_prompt.const.colors import COLOR_GRAY, COLOR_DEFAULT
from wexample_prompt.enums.message_type import MessageType
from wexample_prompt.themes.default.abstract_prompt_theme import AbstractPromptTheme


class DefaultPromptTheme(AbstractPromptTheme):
    def get_color(self, message_type: MessageType) -> str:
        return {
            MessageType.LOG: COLOR_GRAY
        }.get(message_type, COLOR_DEFAULT)
